@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     tape_handling.cpp
- Revision: $Id: tape_handling.cpp 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: tape_handling.cpp 157 2010-09-02 12:14:24Z kulshres $
  Contents: management of tape infos
 
  Copyright (c) Andreas Kowarz, Andrea Walther
@@ -11,9 +11,9 @@
  recipient's acceptance of the terms of the accompanying license file.
  
 ---------------------------------------------------------------------------*/
-#include <taping_p.h>
-#include <checkpointing_p.h>
-#include <revolve.h>
+#include "taping_p.h"
+#include "checkpointing_p.h"
+#include <adolc/revolve.h>
 
 #include <iostream>
 #include <string.h>
@@ -26,6 +26,15 @@
 
 using namespace std;
 
+#ifdef SPARSE
+BEGIN_C_DECLS
+extern void freeSparseJacInfos(double *y, double **B, unsigned int **JP, void *g, 
+			       void *jr1d, int seed_rows, int seed_clms, int depen);
+extern void freeSparseHessInfos(double **Hcomp, double ***Xppp, double ***Yppp, double ***Zppp, 
+				double **Upp, unsigned int **HP,
+				void *g, void *hr, int p, int indep);
+END_C_DECLS
+#endif
 /* vector of tape infos for all tapes in use */
 vector<TapeInfos *> ADOLC_TAPE_INFOS_BUFFER_DECL;
 
@@ -147,15 +156,32 @@ int initNewTape(short tapeID) {
     }
     newTapeInfos->traceFlag=1;
     newTapeInfos->inUse=1;
+#ifdef SPARSE
     newTapeInfos->pTapeInfos.inJacSparseUse=0;
     newTapeInfos->pTapeInfos.inHessSparseUse=0;
     newTapeInfos->pTapeInfos.sJinfos.B=NULL;
     newTapeInfos->pTapeInfos.sJinfos.y=NULL;
+    newTapeInfos->pTapeInfos.sJinfos.g=NULL;
+    newTapeInfos->pTapeInfos.sJinfos.jr1d=NULL;
+    newTapeInfos->pTapeInfos.sJinfos.Seed=NULL;
+    newTapeInfos->pTapeInfos.sJinfos.JP=NULL;
+    newTapeInfos->pTapeInfos.sJinfos.depen=0;
+    newTapeInfos->pTapeInfos.sJinfos.nnz_in=0;
+    newTapeInfos->pTapeInfos.sJinfos.seed_rows=0;
+    newTapeInfos->pTapeInfos.sJinfos.seed_clms=0;
     newTapeInfos->pTapeInfos.sHinfos.Zppp=NULL;
     newTapeInfos->pTapeInfos.sHinfos.Yppp=NULL;
     newTapeInfos->pTapeInfos.sHinfos.Xppp=NULL;
     newTapeInfos->pTapeInfos.sHinfos.Upp=NULL;
     newTapeInfos->pTapeInfos.sHinfos.Hcomp=NULL;
+    newTapeInfos->pTapeInfos.sHinfos.HP=NULL;
+    newTapeInfos->pTapeInfos.sHinfos.g=NULL;
+    newTapeInfos->pTapeInfos.sHinfos.hr=NULL;
+    newTapeInfos->pTapeInfos.sHinfos.nnz_in=0;
+    newTapeInfos->pTapeInfos.sHinfos.indep=0;
+    newTapeInfos->pTapeInfos.sHinfos.p=0;
+#endif
+
     newTapeInfos->stats[OP_BUFFER_SIZE] =
         ADOLC_GLOBAL_TAPE_VARS.operationBufferSize;
     newTapeInfos->stats[LOC_BUFFER_SIZE] =
@@ -308,13 +334,16 @@ TapeInfos *getTapeInfos(short tapeID) {
     ADOLC_TAPE_INFOS_BUFFER.push_back(tapeInfos);
     tapeInfos->traceFlag=1;
     tapeInfos->inUse=0;
+#ifdef SPARSE
     tapeInfos->pTapeInfos.inJacSparseUse=0;
     tapeInfos->pTapeInfos.inHessSparseUse=0;
+#endif
     tapeInfos->tapingComplete = 1;
     read_tape_stats(tapeInfos);
     return tapeInfos;
 }
 
+#ifdef SPARSE
 /* updates the tape infos on sparse Jac for the given ID  */
 void setTapeInfoJacSparse(short tapeID, SparseJacInfos sJinfos) {
     TapeInfos *tapeInfos;
@@ -330,20 +359,32 @@ void setTapeInfoJacSparse(short tapeID, SparseJacInfos sJinfos) {
                 ++tiIter) {
             if ((*tiIter)->tapeID==tapeID) {
                 tapeInfos=*tiIter;
-		    // memory deallocation is missing !!
-		    tapeInfos->pTapeInfos.sJinfos.y=sJinfos.y;
-		    tapeInfos->pTapeInfos.sJinfos.Seed=sJinfos.Seed;
-		    tapeInfos->pTapeInfos.sJinfos.B=sJinfos.B;
-		    tapeInfos->pTapeInfos.sJinfos.JP=sJinfos.JP;
-		    tapeInfos->pTapeInfos.sJinfos.nnz_in=sJinfos.nnz_in;
-		    tapeInfos->pTapeInfos.sJinfos.p=sJinfos.p;
-		    tapeInfos->pTapeInfos.sJinfos.g=sJinfos.g;
-		    tapeInfos->pTapeInfos.sJinfos.jr1d=sJinfos.jr1d;
+		// free memory of tape entry that had been used previously
+		freeSparseJacInfos(tapeInfos->pTapeInfos.sJinfos.y,
+                        tapeInfos->pTapeInfos.sJinfos.B,
+                        tapeInfos->pTapeInfos.sJinfos.JP,
+                        tapeInfos->pTapeInfos.sJinfos.g,
+			tapeInfos->pTapeInfos.sJinfos.jr1d,
+			tapeInfos->pTapeInfos.sJinfos.seed_rows,
+			tapeInfos->pTapeInfos.sJinfos.seed_clms,
+			tapeInfos->pTapeInfos.sJinfos.depen);
+		tapeInfos->pTapeInfos.sJinfos.y=sJinfos.y;
+		tapeInfos->pTapeInfos.sJinfos.Seed=sJinfos.Seed;
+		tapeInfos->pTapeInfos.sJinfos.B=sJinfos.B;
+		tapeInfos->pTapeInfos.sJinfos.JP=sJinfos.JP;
+		tapeInfos->pTapeInfos.sJinfos.depen=sJinfos.depen;
+		tapeInfos->pTapeInfos.sJinfos.nnz_in=sJinfos.nnz_in;
+		tapeInfos->pTapeInfos.sJinfos.seed_clms=sJinfos.seed_clms;
+		tapeInfos->pTapeInfos.sJinfos.seed_rows=sJinfos.seed_rows;
+		tapeInfos->pTapeInfos.sJinfos.g=sJinfos.g;
+		tapeInfos->pTapeInfos.sJinfos.jr1d=sJinfos.jr1d;
             }
         }
     }
 }
+#endif
 
+#ifdef SPARSE
 /* updates the tape infos on sparse Hess for the given ID  */
 void setTapeInfoHessSparse(short tapeID, SparseHessInfos sHinfos) {
     TapeInfos *tapeInfos;
@@ -359,13 +400,24 @@ void setTapeInfoHessSparse(short tapeID, SparseHessInfos sHinfos) {
                 ++tiIter) {
             if ((*tiIter)->tapeID==tapeID) {
                 tapeInfos=*tiIter;
-		    // memory deallocation is missing !!
+		// free memory of tape entry that had been used previously
+                    freeSparseHessInfos(tapeInfos->pTapeInfos.sHinfos.Hcomp, 
+                                        tapeInfos->pTapeInfos.sHinfos.Xppp, 
+                                        tapeInfos->pTapeInfos.sHinfos.Yppp, 
+                                        tapeInfos->pTapeInfos.sHinfos.Zppp, 
+                                        tapeInfos->pTapeInfos.sHinfos.Upp, 
+                                        tapeInfos->pTapeInfos.sHinfos.HP,
+					tapeInfos->pTapeInfos.sHinfos.g, 
+                                        tapeInfos->pTapeInfos.sHinfos.hr, 
+                                        tapeInfos->pTapeInfos.sHinfos.p, 
+                                        tapeInfos->pTapeInfos.sHinfos.indep);	
 		    tapeInfos->pTapeInfos.sHinfos.Hcomp=sHinfos.Hcomp;
 		    tapeInfos->pTapeInfos.sHinfos.Xppp=sHinfos.Xppp;
 		    tapeInfos->pTapeInfos.sHinfos.Yppp=sHinfos.Yppp;
 		    tapeInfos->pTapeInfos.sHinfos.Zppp=sHinfos.Zppp;
 		    tapeInfos->pTapeInfos.sHinfos.Upp=sHinfos.Upp;
 		    tapeInfos->pTapeInfos.sHinfos.HP=sHinfos.HP;
+		    tapeInfos->pTapeInfos.sHinfos.indep=sHinfos.indep;
 		    tapeInfos->pTapeInfos.sHinfos.nnz_in=sHinfos.nnz_in;
 		    tapeInfos->pTapeInfos.sHinfos.p=sHinfos.p;
 		    tapeInfos->pTapeInfos.sHinfos.g=sHinfos.g;
@@ -374,6 +426,7 @@ void setTapeInfoHessSparse(short tapeID, SparseHessInfos sHinfos) {
         }
     }
 }
+#endif
 
 void init() {
     ADOLC_OPENMP_THREAD_NUMBER;
@@ -472,52 +525,26 @@ void cleanUp() {
                 (*tiIter)->tayBuffer = NULL;
             }
 
-
-            if ((*tiIter)->pTapeInfos.sJinfos.B != NULL)
-            {
-                free((char*)  *(*tiIter)->pTapeInfos.sJinfos.B);
-                free((char*)   (*tiIter)->pTapeInfos.sJinfos.B);
-                (*tiIter)->pTapeInfos.sJinfos.B = NULL;
-            }
-            if ((*tiIter)->pTapeInfos.sJinfos.y != NULL)
-            {
-                free((*tiIter)->pTapeInfos.sJinfos.y);
-                (*tiIter)->pTapeInfos.sJinfos.y = NULL;
-            }
-
-            if ((*tiIter)->pTapeInfos.sHinfos.Zppp != NULL)
-            {
-                free((char*) **(*tiIter)->pTapeInfos.sHinfos.Zppp);
-                free((char*)  *(*tiIter)->pTapeInfos.sHinfos.Zppp);
-                free((char*)   (*tiIter)->pTapeInfos.sHinfos.Zppp);
-                (*tiIter)->pTapeInfos.sHinfos.Zppp = NULL;
-            }
-            if ((*tiIter)->pTapeInfos.sHinfos.Yppp != NULL)
-            {
-                free((char*) **(*tiIter)->pTapeInfos.sHinfos.Yppp);
-                free((char*)  *(*tiIter)->pTapeInfos.sHinfos.Yppp);
-                free((char*)   (*tiIter)->pTapeInfos.sHinfos.Yppp);
-                (*tiIter)->pTapeInfos.sHinfos.Yppp = NULL;
-            }
-            if ((*tiIter)->pTapeInfos.sHinfos.Xppp != NULL)
-            {
-                free((char*) **(*tiIter)->pTapeInfos.sHinfos.Xppp);
-                free((char*)  *(*tiIter)->pTapeInfos.sHinfos.Xppp);
-                free((char*)   (*tiIter)->pTapeInfos.sHinfos.Xppp);
-                (*tiIter)->pTapeInfos.sHinfos.Yppp = NULL;
-            }
-            if ((*tiIter)->pTapeInfos.sHinfos.Upp != NULL)
-            {
-                free((char*)  *(*tiIter)->pTapeInfos.sHinfos.Upp);
-                free((char*)   (*tiIter)->pTapeInfos.sHinfos.Upp);
-                (*tiIter)->pTapeInfos.sHinfos.Upp = NULL;
-            }
-            if ((*tiIter)->pTapeInfos.sHinfos.Hcomp != NULL)
-            {
-                free((char*)  *(*tiIter)->pTapeInfos.sHinfos.Hcomp);
-                free((char*)   (*tiIter)->pTapeInfos.sHinfos.Hcomp);
-                (*tiIter)->pTapeInfos.sHinfos.Hcomp = NULL;
-            }
+#ifdef SPARSE
+	    freeSparseJacInfos((*tiIter)->pTapeInfos.sJinfos.y,
+			       (*tiIter)->pTapeInfos.sJinfos.B,
+			       (*tiIter)->pTapeInfos.sJinfos.JP,
+			       (*tiIter)->pTapeInfos.sJinfos.g,
+			       (*tiIter)->pTapeInfos.sJinfos.jr1d,
+			       (*tiIter)->pTapeInfos.sJinfos.seed_rows,
+			       (*tiIter)->pTapeInfos.sJinfos.seed_clms,
+			       (*tiIter)->pTapeInfos.sJinfos.depen);
+	    freeSparseHessInfos((*tiIter)->pTapeInfos.sHinfos.Hcomp, 
+				(*tiIter)->pTapeInfos.sHinfos.Xppp, 
+				(*tiIter)->pTapeInfos.sHinfos.Yppp, 
+				(*tiIter)->pTapeInfos.sHinfos.Zppp, 
+				(*tiIter)->pTapeInfos.sHinfos.Upp, 
+				(*tiIter)->pTapeInfos.sHinfos.HP,
+				(*tiIter)->pTapeInfos.sHinfos.g, 
+				(*tiIter)->pTapeInfos.sHinfos.hr, 
+				(*tiIter)->pTapeInfos.sHinfos.p, 
+				(*tiIter)->pTapeInfos.sHinfos.indep);	
+#endif
 
             /* remove "main" tape files if not all three have been written */
             int filesWritten = (*tiIter)->stats[OP_FILE_ACCESS] +
@@ -611,6 +638,26 @@ int removeTape(short tapeID, short type) {
     }
 
     freeTapeResources(tapeInfos);
+#ifdef SPARSE
+    freeSparseJacInfos(tapeInfos->pTapeInfos.sJinfos.y,
+		       tapeInfos->pTapeInfos.sJinfos.B,
+		       tapeInfos->pTapeInfos.sJinfos.JP,
+		       tapeInfos->pTapeInfos.sJinfos.g,
+		       tapeInfos->pTapeInfos.sJinfos.jr1d,
+		       tapeInfos->pTapeInfos.sJinfos.seed_rows,
+		       tapeInfos->pTapeInfos.sJinfos.seed_clms,
+		       tapeInfos->pTapeInfos.sJinfos.depen);
+    freeSparseHessInfos(tapeInfos->pTapeInfos.sHinfos.Hcomp, 
+			tapeInfos->pTapeInfos.sHinfos.Xppp, 
+			tapeInfos->pTapeInfos.sHinfos.Yppp, 
+			tapeInfos->pTapeInfos.sHinfos.Zppp, 
+			tapeInfos->pTapeInfos.sHinfos.Upp, 
+			tapeInfos->pTapeInfos.sHinfos.HP,
+			tapeInfos->pTapeInfos.sHinfos.g, 
+			tapeInfos->pTapeInfos.sHinfos.hr, 
+			tapeInfos->pTapeInfos.sHinfos.p, 
+			tapeInfos->pTapeInfos.sHinfos.indep);	
+#endif
     ADOLC_OPENMP_RESTORE_THREAD_NUMBER;
 
     if (type == ADOLC_REMOVE_COMPLETELY) {
