@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     taping.c
- Revision: $Id: taping.c 42 2009-07-15 18:37:17Z awalther $
+ Revision: $Id: taping.c 116 2010-07-20 12:29:32Z awalther $
  Contents: all C functions directly accessing at least one of the four tapes
            (operations, locations, constants, value stack)
 
@@ -17,8 +17,8 @@
 #include <math.h>
 #include <string.h>
 
-#include <oplate.h>
-#include <taping_p.h>
+#include <adolc/oplate.h>
+#include "taping_p.h"
 
 /*--------------------------------------------------------------------------*/
 /* Tape identification (ADOLC & version check) */
@@ -300,10 +300,10 @@ void printError() {
 
 /* the base names of every tape type */
 const char *tapeBaseNames[] = {
-            ADOLC_LOCATIONS_NAME,
-            ADOLC_VALUES_NAME,
-            ADOLC_OPERATIONS_NAME,
-            ADOLC_TAYLORS_NAME
+            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_LOCATIONS_NAME,
+            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_VALUES_NAME,
+            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_OPERATIONS_NAME,
+            ADOLC_TAPE_DIR PATHSEPARATOR ADOLC_TAYLORS_NAME
         };
 
 /****************************************************************************/
@@ -1556,8 +1556,8 @@ void end_sweep() {
         fclose(ADOLC_CURRENT_TAPE_INFOS.val_file);
         ADOLC_CURRENT_TAPE_INFOS.val_file = NULL;
     }
-    if (ADOLC_CURRENT_TAPE_INFOS.deg_save > 0) releaseTape(1); /* keep value stack */
-    else releaseTape(0); /* no value stack */
+    if (ADOLC_CURRENT_TAPE_INFOS.deg_save > 0) releaseTape(); /* keep value stack */
+    else releaseTape(); /* no value stack */
 }
 
 /* --- Operations --- */
