@@ -1,7 +1,7 @@
 /*----------------------------------------------------------------------------
  ADOL-C -- Automatic Differentiation by Overloading in C++
  File:     taping_p.h
- Revision: $Id: taping_p.h 91 2010-02-24 07:56:58Z awalther $
+ Revision: $Id: taping_p.h 157 2010-09-02 12:14:24Z kulshres $
  Contents: declarations for used by taping routines
  
  Copyright (c) Andreas Kowarz
@@ -14,8 +14,8 @@
 #if !defined(ADOLC_TAPING_P_H)
 #define ADOLC_TAPING_P_H 1
 
-#include <common.h>
-#include <taping.h>
+#include <adolc/common.h>
+#include <adolc/taping.h>
 #include <errno.h>
 
 BEGIN_C_DECLS
@@ -121,17 +121,18 @@ extern void *failAdditionalInfo6;
 /* tape information                                                         */
 /****************************************************************************/
 
+#ifdef SPARSE
 typedef struct SparseJacInfos {
-    void *g;
-    void *jr1d;
+  void *g;
+  void *jr1d;
 
-    double *y;
-    double **Seed;
-    double **B;
+  double *y;
+  double **Seed;
+  double **B;
 
-    unsigned int **JP;
+  unsigned int **JP;
 
-    int nnz_in, p;
+  int depen, nnz_in, seed_clms, seed_rows;
 } SparseJacInfos;
 
 typedef struct SparseHessInfos {
@@ -146,8 +147,9 @@ typedef struct SparseHessInfos {
   
     unsigned int **HP;
 
-    int nnz_in, p;
+  int nnz_in, indep, p;
 } SparseHessInfos;
+#endif
 
 typedef struct PersistantTapeInfos { /* survive tape re-usage */
     int forodec_nax, forodec_dax;
@@ -159,6 +161,7 @@ typedef struct PersistantTapeInfos { /* survive tape re-usage */
     int *jacSolv_ci;
     int jacSolv_nax, jacSolv_modeold, jacSolv_cgd;
 
+#ifdef SPARSE
     /* sparse derivative matrices */
 
     int inJacSparseUse;
@@ -171,6 +174,7 @@ typedef struct PersistantTapeInfos { /* survive tape re-usage */
     /* sparse Hessian matrices */
 
     SparseHessInfos sHinfos;
+#endif
 
     /* file names */
 
@@ -362,11 +366,13 @@ TapeInfos *getTapeInfos(short tapeID);
 /* updates the tape infos for the given ID - a tapeInfos struct is created
  * and registered if non is found but its state will remain "not in use" */
 
+#ifdef SPARSE
 void setTapeInfoJacSparse(short tapeID, SparseJacInfos sJinfos);
 /* updates the tape infos on sparse Jac for the given ID */
 
 void setTapeInfoHessSparse(short tapeID, SparseHessInfos sHinfos);
 /* updates the tape infos n sparse Hess for the given ID */
+#endif
 
 void take_stock();
 /* record all existing adoubles on the tape
